<?php

defined('_JEXEC') or die;

/**
 * Class CInstallerHelperRequirements
 *
 * @since  1.0.0
 */
class CInstallerHelperRequirements
{
	/**
	 * The instance (singleton)
	 *
	 * @var    CInstallerHelperRequirements
	 *
	 * @since  1.0.0
	 */
	private static $instance = null;

	/**
	 * The product
	 *
	 * @var    CProductInterface|null
	 *
	 * @since  1.0.0
	 */
	protected $product = null;


	/**
	 * The requirements
	 *
	 * @var    Requirement[]  An array of requirements
	 *
	 * @since  1.0.0
	 */
	protected $requirements = array();

	/**
	 * CInstallerHelperRequirements constructor.
	 *
	 * @param  CProductInterface  $product  The product
	 *
	 * @since  1.0.0
	 */
	private function __construct(CProductInterface $product)
	{
		$this->product = $product;
	}

	/**
	 * Get the instance of the requirements
	 *
	 * @param CProductInterface $product
	 *
	 * @return CInstallerHelperRequirements
	 *
	 * @since  1.0.0
	 */
	public static function getInstance(CProductInterface $product)
	{
		if (self::$instance == null)
		{
			self::$instance = new CInstallerHelperRequirements($product);

			self::$instance->init();
		}

		return self::$instance;
	}

	/**
	 * Init the Requirements
	 *
	 * @since   1.0.0
	 */
	public function init()
	{
		/** @var CInstallerRequirements $requirements */
		$minRequirements = $this->product->getRequirements();
		$result = array();

		$it = $minRequirements->getIterator();

		foreach ($it as $req)
		{
			$method = 'get' . ucfirst($req->name);

			if (method_exists($this, $method))
			{
				$result[] = $this->$method($req);
			}
		}

		$this->requirements = $result;
	}

	/**
	 * Get the Requirements
	 *
	 * @return  Requirement[]  An array of requirements
	 *
	 * @since   1.0.0
	 */
	public function get()
	{
		return $this->requirements;
	}

	/**
	 * Get the requirement for PHP Min
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	protected function getPhpMin($requirement)
	{
		$version = phpversion();

		if (version_compare($version, $requirement->target, '<='))
		{
			$requirement->isFullfilled = false;
		}

		$requirement->currentVersion = $version;

		return $requirement;
	}

	/**
	 * Get the requirement for PHP Min
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getPhpOptimum($requirement)
	{
		return $this->getPhpMin($requirement);
	}

	/**
	 * Get the requirement for Curl
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getPhpCurl($requirement)
	{
		$version = curl_version()['version'];

		if (version_compare($version, $requirement->target, 'LT'))
		{
			$requirement->isFullfilled = false;
		}

		$requirement->currentVersion = $version;

		return $requirement;
	}

	/**
	 * Get the requirement for Joomla minimum version
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getJoomlaMin($requirement)
	{
		$requirement->currentVersion = JVERSION;

		if (version_compare(JVERSION, $requirement->target, '<'))
		{
			$requirement->isFullfilled = false;
		}

		return $requirement;
	}

	/**
	 * Get the requirement for Database server
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getDatabaseServer($requirement)
	{
		$dbType = JFactory::getDbo()->getServerType();

		if (!in_array($dbType, $requirement->target))
		{
			$requirement->isFullfilled = false;
		}

		$requirement->currentVersion = $dbType;

		return $requirement;
	}

	/**
	 * Get the requirement for MySQL minimum version
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getMysqlMin($requirement)
	{
		$db = JFactory::getDbo();

		$version = $db->getConnection()->server_info;

		if (version_compare($version, $requirement->target, '<'))
		{
			$requirement->isFullfilled = false;
		}

		$requirement->currentVersion = $version;

		return $requirement;
	}

	/**
	 * Get the requirement for MySQL optimum version
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getMysqlOptimum($requirement)
	{
		return $this->getMysqlMin($requirement);
	}

	/**
	 * Get the requirement for the mysql drivers
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getMysqlDriver($requirement)
	{
		$db = JFactory::getDbo();

		$driver = $db->getName();

		if (!in_array($driver, $requirement->target))
		{
			$requirement->isFullfilled = false;
		}

		$requirement->currentVersion = $driver;

		return $requirement;
	}

	/**
	 * Get the requirement for the php module drivers
	 *
	 * @param   Requirement  $requirement  The requirement
	 *
	 * @return  Requirement
	 *
	 * @since   1.0.0
	 */
	public function getPhpModules($requirement)
	{
		$modules = get_loaded_extensions();

		foreach ($requirement->target as $target)
		{
			if (!in_array($target, $modules))
			{
				$requirement->isFullfilled = false;

				break;
			}
		}

		$requirement->currentVersion = implode(', ', $modules);

		return $requirement;
	}
}
